/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Creative Commons
 * Attribution-NonCommercial-ShareAlike Version 3.0, as published
 * by Creative Commons. See 
 * http://creativecommons.org/licenses/by-nc-sa/3.0/legalcode for
 * more details. */ 
 
 // nsIWebProgressListener implementation to monitor activity in the browser.
function WebProgressListener() {
}
WebProgressListener.prototype = {
  _requestsStarted: 0,
  _requestsFinished: 0,

  // We need to advertize that we support weak references.  This is done simply
  // by saying that we QI to nsISupportsWeakReference.  XPConnect will take
  // care of actually implementing that interface on our behalf.
  QueryInterface: function(iid) {
    if (iid.equals(Components.interfaces.nsIWebProgressListener) ||
        iid.equals(Components.interfaces.nsISupportsWeakReference) ||
        iid.equals(Components.interfaces.nsISupports))
      return this;
    
    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  // This method is called to indicate state changes.
  onStateChange: function(webProgress, request, stateFlags, status) {
    const WPL = Components.interfaces.nsIWebProgressListener;

    var progress = document.getElementById("browser-progress");

    if (stateFlags & WPL.STATE_IS_REQUEST) {
      if (stateFlags & WPL.STATE_START) {
        this._requestsStarted++;
      } else if (stateFlags & WPL.STATE_STOP) {
        this._requestsFinished++;
      }
      if (this._requestsStarted > 1) {
        var value = (100 * this._requestsFinished) / this._requestsStarted;
        progress.setAttribute("mode", "determined");
        progress.setAttribute("value", value + "%");
      }
    }

    if (stateFlags & WPL.STATE_IS_NETWORK) {
      var stop = document.getElementById("stop-button");
	  var reload = document.getElementById("reload-button");
	  var browserStatusbar = document.getElementById("browser-statusbar");
      if (stateFlags & WPL.STATE_START) {
		stop.setAttribute("hidden",false);
		reload.setAttribute("hidden",true);
		browserStatusbar.setAttribute("hidden", false);
		//stop.setAttribute("mode", "stop");
		//stop.setAttribute("oncommand", "mybrowser.stop();");
		//browserStatusbar.setAttribute("hidden",false);
        //progress.setAttribute("style", "");
      } else if (stateFlags & WPL.STATE_STOP) {
		stop.setAttribute("hidden",true);
		reload.setAttribute("hidden",false);
		browserStatusbar.setAttribute("hidden", true);
		/*browserStatusbar.setAttribute("hidden",true);
		stop.setAttribute("mode", "reload");
		stop.setAttribute("oncommand","mybrowser.reload();");*/
        //progress.setAttribute("style", "display: none");
		//mdc.showThrobber("internet", false);
        this.onStatusChange(webProgress, request, 0, "Done");
        this._requestsStarted = this._requestsFinished = 0;
      }
    }
  },

  // This method is called to indicate progress changes for the currently
  // loading page.
  onProgressChange: function(webProgress, request, curSelf, maxSelf,
                             curTotal, maxTotal) {
    if (this._requestsStarted == 1) {
      var progress = document.getElementById("browser-progress");
      if (maxSelf == -1) {
        progress.setAttribute("mode", "undetermined");
      } else {
        progress.setAttribute("mode", "determined");
        progress.setAttribute("value", ((100 * curSelf) / maxSelf) + "%");
      }
    }
  },

  // This method is called to indicate a change to the current location.
  onLocationChange: function(webProgress, request, location) {
    mdc.clearTree("webscraper_rows");
	document.getElementById("webscraper_splitter").setAttribute("state","collapsed");
	mybrowser.loadWebScraper(location.spec);
	var urlbar = document.getElementById("urlbar");
	urlbar.value = location.spec;

    var browser = document.getElementById("browser");
    var back = document.getElementById("back-button");
    var forward = document.getElementById("forward-button");	
	
	
    back.setAttribute("disabled", !browser.canGoBack);
    forward.setAttribute("disabled", !browser.canGoForward);
  },

  // This method is called to indicate a status changes for the currently
  // loading page.  The message is already formatted for display.
  onStatusChange: function(webProgress, request, status, message) {
    var status = document.getElementById("browser-statusmessage");
    status.setAttribute("value", message);
  },

  // This method is called when the security state of the browser changes.
  onSecurityChange: function(webProgress, request, state) {
    const WPL = Components.interfaces.nsIWebProgressListener;

    var sec = document.getElementById("security");

    if (state & WPL.STATE_IS_INSECURE) {
      sec.setAttribute("style", "display: none");
    } else {
      var level = "unknown";
      if (state & WPL.STATE_IS_SECURE) {
        if (state & WPL.STATE_SECURE_HIGH)
          level = "high";
        else if (state & WPL.STATE_SECURE_MED)
          level = "medium";
        else if (state & WPL.STATE_SECURE_LOW)
          level = "low";
      } else if (state & WPL_STATE_IS_BROKEN) {
        level = "mixed";
      }
      sec.setAttribute("value", "Security: " + level);
      sec.setAttribute("style", "");
    }
  }
};
var listener;
var mybrowser = {
	homePage: "http://www.google.ch/firefox",
	
	go: function() {
	  var urlbar = document.getElementById("urlbar");
	  var browser = document.getElementById("browser");

	  browser.loadURI(urlbar.value, null, null);
	},
	
	goHome: function(){
		var browser = document.getElementById("browser");
		browser.loadURI(mybrowser.homePage, null, null);
	},
	
	goTo: function( aURL ){
		var browser = document.getElementById("browser");
		browser.loadURI(aURL, null, null);
	},

	back: function() {
	  var browser = document.getElementById("browser");
	  browser.stop();
	  browser.goBack();
	},

	forward: function() {
	  var browser = document.getElementById("browser");
	  browser.stop();
	  browser.goForward();
	},

	reload: function() {
	  var browser = document.getElementById("browser");
	  browser.reload();
	},

	stop: function() {
	  var browser = document.getElementById("browser");
	  browser.stop();
	},

	onload: function() {
	  var urlbar = document.getElementById("urlbar");
	  urlbar.value = mybrowser.homePage;

	  listener = new WebProgressListener();

	  var browser = document.getElementById("browser");
	  browser.focus();
	  browser.addProgressListener(listener,
		Components.interfaces.nsIWebProgress.NOTIFY_ALL);

	  mybrowser.go();
	  mdc.browser_init = true;	//no onload at 2nd load
	},


	loadWebScraper: function(url){
		var worker = new Worker("webscraper.js");
		worker.onmessage = function(event) {
			mybrowser.displayResults(event.data);
		};
		worker.postMessage(url);
	},
	
	/**
	 * Displays the results from the webscraper web worker.
	 * @param array data An array of the results
	*/
	displayResults: function(data){
		var splitter = document.getElementById("webscraper_splitter");
		var listbox = document.getElementById("webscraper_listbox");
		//var urlbar = document.getElementById("urlbar");
		if(data){
			if(data.length>0){
				if(splitter.getAttribute("state") =="collapsed"){
					splitter.setAttribute("state", "open");
				}
				for(var i=0; i<data.length;++i){
					var filename = unescape(data[i]).split("/");
					filename = filename[filename.length-1];
					var listbox = document.getElementById("webscraper_rows");
					var item = document.createElement("richlistitem");
						item.setAttribute("id", data[i])
					var label = document.createElement("label");
						label.setAttribute("value", filename);
						label.setAttribute("flex", "1");
					var button = document.createElement("button");
						button.setAttribute("label","Download");
						button.setAttribute("id", data[i]);
						button.setAttribute("height","1px");
						button.setAttribute("onclick","var file = mdc.filePicker('save', null, '"+filename+"', 'mp3'); mdc.downloadBinaryFile('"+unescape(data[i])+"', file);");
					item.appendChild(label);	//mdc.downloadBinaryFile('"+data[i]+"', "+file+");
					item.appendChild(button);
					listbox.appendChild(item);
				}
			}
		}
	},
};

addEventListener("load", mybrowser.onload, false);