/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Creative Commons
 * Attribution-NonCommercial-ShareAlike Version 3.0, as published
 * by Creative Commons. See 
 * http://creativecommons.org/licenses/by-nc-sa/3.0/legalcode for
 * more details. */ 
 
 // ------------------------------------------------------------------
// use helper functions to hook up the mdc object so "this"
// works in the explorer object
// ------------------------------------------------------------------

function mdc_startup() {
  mdc.startup();
}

/*function mdc_shutdown() {
  mdc.shutdown();
}*/


// ------------------------------------------------------------------
// attach to window events so mdc object can startup / shutdown
// ------------------------------------------------------------------

//window.addEventListener("load", mdc_startup, false);
//window.addEventListener("unload", mdc_shutdown, false);


// ------------------------------------------------------------------
// mdc object
// ------------------------------------------------------------------

var mdc = {
	currentTitle: "",
	currentType: "",
	currentId: "",
	
	browser_init: false,
	radio_init: false,
	
	initialized : false,

	
	searchEngine: "search",	

	startup : function() {
		/*Perform some startup thingies (strings, check for updates and stuff)*/
		if (this.initialized)
			return;
		this.initialized = true;

		var self = this;

		dump("Welcome to the Music Download Center");
		
		var strings = document.getElementById("extendedlibrary-strings");
			this.searchcomplete = strings.getString('extendedlibrary.searchcomplete');
			this.updateAvailable = strings.getString('extendedlibrary.updateAvailable');
			this.message = strings.getString('extendedlibrary.message');
			this.URLCopied = strings.getString('extendedlibrary.URLCopied');
			this.activeDownloads = strings.getString('extendedlibrary.activeDownloads');
			
		this.updateCheck();
		
		document.getElementById('searchbar-textbox').focus();
		serviceman.restoreDB();
		serviceman.restoreDBStructure();
		serviceman.compareSources();
		var showDialog = this.getPref("application.firstrun.enabled", "bool");
		if(showDialog){
			mdc.openWindow('chrome://mdc/content/firstrun.xul', '_blank', 'chrome,extrachrome,menubar,resizable,scrollbars,status,toolbar')
		}
	},
  
	clearTree: function( tree_id ){
		/*Basic function that clears trees. Takes the tree's ID as parameter*/
		var element = document.getElementById(tree_id);
		if(element){
			while(element.hasChildNodes()){
    			element.removeChild(element.firstChild);
			}
		}
	},
	
	downloadBinaryFile: function(source, target) {
		/*Download a file with the built-in download manager. Takes an nsIURL and an nsIFile*/
		var tr = Components.classes["@mozilla.org/transfer;1"].createInstance(Components.interfaces.nsITransfer);
		var ioService  = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
		var uri = ioService.newURI(unescape(source), null , null);
		var url = uri.QueryInterface(Components.interfaces.nsIURL);	
		var fileURI = ioService.newFileURI(target);
		var nsIWBP = Components.interfaces.nsIWebBrowserPersist;
		var persist = Components.classes['@mozilla.org/embedding/browser/nsWebBrowserPersist;1'].createInstance(Components.interfaces.nsIWebBrowserPersist);
		var flags = nsIWBP.PERSIST_FLAGS_NO_CONVERSION |
			nsIWBP.PERSIST_FLAGS_REPLACE_EXISTING_FILES |
			nsIWBP.PERSIST_FLAGS_BYPASS_CACHE;
		persist.persistFlags = flags;
		tr.init(uri , fileURI , target.leafName, null , null , null , null )
		persist.progressListener = tr;
		persist.saveURI(uri, null, null, null, "", fileURI);
	},
	
	getFile: function(source) {
		/*Get the source code of a website specified with 'source'*/
		dump("\nGetting File from "+source);
		var req = new XMLHttpRequest();
		req.open('GET', source, false);
		try{
			req.send(null);
			if(req.status == 200){
				var aMessage = req.responseText;
			}
			return aMessage;
		}
		catch(e){
			alert("Error. Internet is gone.");
			return;
		}
		
	},
	
	filePicker: function(mode, defaultDir, defaultName, defaultExtension) {
		/*Display a filePicker. Fully customizeable. See the Mozilla Developer Network
		for more Help.
			mode	save: Used for saving one file
					open: Used for opening a single file
					getFolder: Used for getting a folder
					openMultiple: Used for opening multiple files at once*/
		const nsIFilePicker = Components.interfaces.nsIFilePicker;
		var fp = Components.classes["@mozilla.org/filepicker;1"]
						.createInstance(nsIFilePicker);
		switch(mode){
			case "save":
				fp.init(window, this.message, nsIFilePicker.modeSave);
				break;
			case "open":
				fp.init(window, this.message, nsIFilePicker.modeOpen);
				break;
			case "getFolder":
				fp.init(window,this.message, nsIFilePicker.modeGetFolder);
				break;
			case "openMultiple":
				fp.init(window, this.message, nsIFilePicker.modeOpenMultiple);
				break;	
		}
		fp.appendFilters(nsIFilePicker.filterAll);
		fp.appendFilter("MPEG Audio Layer 3","*.mp3");
		fp.defaultString = defaultName;
		fp.defaultExtension = defaultExtension;
		fp.displayDirectory = defaultDir;
		var rv = fp.show();
		if (rv == nsIFilePicker.returnOK || rv == nsIFilePicker.returnReplace) {
		  var file = fp.file;
		  return file;
		}
		if (rv == nsIFilePicker.returnCancel) {
		  return "";
		}
	},
	
	removeEntities: function(aString){
		/*Manually removes HTML Entities from a string. Crappy work.*/
		return aString.replace(/\&\#39\;/g,"'").replace(/\&\#039\;/g,"'").replace(/\&amp\;/g,"&").replace(/\&quot\;/g,"'");
	},
	
	queryDatabase: function(query, column){
		/*make database Queries through this function easy to use*/
		dump("\nDBQuery: "+query);
		var file = Components.classes["@mozilla.org/file/directory_service;1"]
                     .getService(Components.interfaces.nsIProperties)
                     .get("ProfD", Components.interfaces.nsIFile);
		file.append("services.db");

		var storageService = Components.classes["@mozilla.org/storage/service;1"]
								.getService(Components.interfaces.mozIStorageService);
		var dbConn = storageService.openUnsharedDatabase(file); // Will also create the file if it does not exist
		try{
			var statement = dbConn.createStatement(query);
		}
		catch(err){
			alert("Error creating Database Query: "+query+"\nError description is"+err.description);
		}
		var result = new Array();
		var counter=0;
		
		try {
			while (statement.executeStep()) {
				result[counter] = statement.getString(column);
				counter++;
			}
		}
		finally {
			statement.reset();
		}
		return result;
	},
	
	popup: function(title, msg) {
		/*Shows a notification on *most* platforms. Looks crappy on Darwin. To be removed.*/
		var image = null
		var win = Components.classes['@mozilla.org/embedcomp/window-watcher;1']
					  .getService(Components.interfaces.nsIWindowWatcher)
					  .openWindow(null, 'chrome://global/content/alerts/alert.xul',
								  '_blank', 'chrome,titlebar=no,popup=yes', null)
		win.arguments = [image, title, msg, false, '']
	},
	
	updateCheck: function(){
		/*Check for updates o Linux because automatic update doesn't
		work (or I am too stupid).*/
		var newVersion = this.getFile("http://dieners.di.funpic.de/extlib/version.txt");
		var OS = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULRuntime).OS;
		var thisVersion = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULAppInfo).version;
		if (OS=="Linux" && thisVersion < newVersion){
			alert(mdc.updateAvailable);
			this.openlink("http://code.google.com/p/musicdownloadcenter/downloads/list?can=2&q=OpSys%3DLinux+&colspec=Filename+Summary+Uploaded+Size+DownloadCount");
		}
	},
	
	openlinkExternal: function(url) {
		/*Open a link in the default browser.*/
		var ioService = Components.classes["@mozilla.org/network/io-service;1"]
							  .getService(Components.interfaces.nsIIOService);
		var uri = ioService.newURI(url, null, null);
		var extProtocolSvc = Components.classes["@mozilla.org/uriloader/external-protocol-service;1"]
								   .getService(Components.interfaces.nsIExternalProtocolService);
		extProtocolSvc.loadUrl(uri);
	},
	
	openlinkInternal: function(url) {
		if(url)
		{
			mdc.changeTab(document.getElementById("tab_internet"));
			document.getElementById("urlbar").value = url;
			mybrowser.go();
		}
	},
	
	checkForUpdates: function()	{
		/*Check for updates with the built-in updater*/
	  var um = 
		  Components.classes["@mozilla.org/updates/update-manager;1"].
		  getService(Components.interfaces.nsIUpdateManager);
	  var prompter = 
		  Components.classes["@mozilla.org/updates/update-prompt;1"].
		  createInstance(Components.interfaces.nsIUpdatePrompt);

	  // If there's an update ready to be applied, show the "Update Downloaded"
	  // UI instead and let the user know they have to restart the browser for
	  // the changes to be applied. 
	  if (um.activeUpdate && um.activeUpdate.state == "pending")
		prompter.showUpdateDownloaded(um.activeUpdate);
	  else
		prompter.checkForUpdates();
	},
	
	share: function( platform ){
		/*Share the currently playing track*/
		var url = document.getElementById('player').src;
		if (url != "chrome://mdc/content/main.xul"){
			var pattern = /audioUrl=(.*)&autoPlay/g;
			while (Ergebnis = pattern.exec(url)){
				url = Ergebnis[1];
			}
			url = url.replace(/\/;$/g,'');			
			switch(platform){
				case "Facebook":
					mdc.openlinkInternal("http://www.facebook.com/sharer.php?u="+url+"&t=Check this!");
					break;
				case "Twitter":
					mdc.openlinkInternal("http://twitter.com/?status=Listening%20to%20"+url+"%20with%20%23musicdownloadcenter");
					break;
				case "MySpace":
					mdc.openlinkInternal("http://www.myspace.com/Modules/PostTo/Pages/?u="+encodeURIComponent(url));
					break;
				case "Link":
					Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper).copyString(decodeURI(unescape(url)));
					mdc.popup("Music Download Center", mdc.URLCopied);
			}
			
		}
	},
	
	openWindow: function(url, name, features) {
		var win = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator).getMostRecentWindow(name);
		if (win) {
			win.focus();
			win.getAttention();
		} else {
			window.open(url, name, 'chrome,extrachrome,centerscreen,menubar,resizable,scrollbars,status,toolbar');
		}//PREFS: "chrome,titlebar,toolbar,centerscreen,dialog=no"
	},
	
	bookmarkCurrentSong: function(){
		var url = document.getElementById('player').src;
		if (url != "chrome://mdc/content/main.xul"){
			var pattern = /audioUrl=(.*mp3)&autoPlay/g;
			while (Ergebnis = pattern.exec(url)){
				url = Ergebnis[1];
			}
			alert(url);
			//mdc.queryDatabase("INSERT INTO 'bookmarks' ('id', 'name', 'url', 'played') VALUES ('"+mdc.currentId+"','"+mdc.currentName+"','"+url+"','1')",0);
		}
	},

	shutdown : function() {
		/*BLERGHHHHHH DON'T READ THIS PART!!!!*/
		if (Components.classes["@mozilla.org/download-manager;1"].getService(Components.interfaces.nsIDownloadManager).activeDownloadCount>0){
			alert(this.activeDownloads);
			return false;
		}
		else{
			return true;
		}
	},
	
	play: function( url, title, type, id){
		/*Play file or stream*/
		var player = document.getElementById("player");
		player.src = 'http://dieners.di.funpic.de/mdc/3523697345-audio-player.swf?audioUrl='+url+'&autoPlay=true';
		player.setAttribute("pluginspage","http://get.adobe.com/flashplayer/");
		document.title = "Music Download Center - "+title;
		document.getElementById("faceplate").value = title;
		mdc.currentType = type;
		mdc.currentId = id;
	},
	
	donate: function(){
		/*Donate plz*/
		var link = "https://www.paypal.com/cgi-bin/webscr?cmd=_donations&business=9BR9DTTMJ4LE2&lc=CH&item_name=Lukas%20Diener&currency_code=CHF&bn=PP%2dDonationsBF%3abtn_donate_SM%2egif%3aNonHosted";
		mdc.openlinkInternal(link);
	},
	
	getLyrics: function( artist, title ){
		/*Nothing to do yet.*/
		var url = 'http://www.absolutelyrics.com/lyrics/view/'+artist+'/'+title+'/';
	},

	getPref: function( name, mode ){
		var prefs = Components.classes["@mozilla.org/preferences-service;1"]
							.getService(Components.interfaces.nsIPrefService);
		var temp = name.split('.');
		var branch = "";
		for(var i=0; i<temp.length-1; ++i){
			branch += temp[i]+".";
		}
		prefs = prefs.getBranch(branch);
		switch(mode){
			case "bool":
				var pref = prefs.getBoolPref(temp[temp.length-1]);
				break;
			case "int":
				var pref = prefs.getIntPref(temp[temp.length-1]);
				break;
			case "char":
				var pref = prefs.getCharPref(temp[temp.length-1]);
				break;
		}
		return pref; 
	},
	
	showThrobber: function( component, value ){
		if(value){document.getElementById('throbber_'+component).hidden = false}
		else{document.getElementById('throbber_'+component).hidden = true};
	},
	
	hideNavBarElements: function (tabItem){
		var tabId = tabItem.id;
		switch(tabId){
			case 'tab_search':
				document.getElementById('location-bar').collapsed= true;
				document.getElementById('search-container').collapsed= false;
				//document.getElementById("search-stop").hidden = false;
				//document.getElementById("clear-all").hidden = false;
				break;
			case 'tab_internet':
				document.getElementById('location-bar').collapsed= false;
				document.getElementById('location-bar').focus();
				document.getElementById('search-container').collapsed= false;
				//document.getElementById("search-stop").hidden = true;
				//document.getElementById("clear-all").hidden = true;
				break;
			case 'tab_radio':
				document.getElementById('location-bar').collapsed= true;
				document.getElementById('search-container').collapsed= false;
				//document.getElementById("search-stop").hidden = true;
				//document.getElementById("clear-all").hidden = false;
				break;
		}
		this.selectSearchEngine(tabId.replace("tab_",""));
	},
	
	selectSearchEngine: function( name ) {
		this.searchEngine = name;
		switch (name){
			case "internet":
				var icon = "http://www.google.com/favicon.ico";
				break;
			case "search":
				var icon = "chrome://mdc/skin/icons/icon16.ico";
				break;
			case "radio":
				var icon = "http://o.aolcdn.com/shoutcast/images/sc_favicon.ico";
				break;
		}
		document.getElementById('search-engine-picker').style.listStyleImage = "url('"+icon+"')";
	},
	
	globalSearch: function( searchterm ){
		document.getElementById("searchbar-textbox").value = searchterm;
		var name = this.searchEngine;
		switch(name){
			case "search":
				this.changeTab(document.getElementById("tab_search"));
				search.searchCommand( searchterm );
				break;
			case "internet":
				this.changeTab(document.getElementById("tab_internet"));
				mybrowser.goTo('http://www.google.ch/search?q='+escape(searchterm));
				break;
			case "radio":
				this.changeTab(document.getElementById("tab_radio"));
				radio.fullSearch(searchterm);
				break;
		}
	},
	
	globalClear: function(){
		var currentTab = this.getCurrentTab();
		document.getElementById('searchbar-textbox').value = '';
		//mdc.clearTree(currentTab+'_treechildren');
	},
	
	changeTab: function(tabItem){
		document.getElementById('tabbrowser-tabpanels').selectedItem = tabItem;
		document.getElementById('tabbrowser-tabs').selectedItem = tabItem;
		mdc.hideNavBarElements(tabItem);
	},
	
	getCurrentTab: function(){
		var id = document.getElementById('tabbrowser-tabs').selectedItem.id;
		return id.replace("tab_","");
	},
	
	restart: function(){
		var appStartup = Components.classes["@mozilla.org/toolkit/app-startup;1"]
                     .getService(Components.interfaces.nsIAppStartup);

		appStartup.quit(Components.interfaces.nsIAppStartup.eRestart |
         		        Components.interfaces.nsIAppStartup.eAttemptQuit);
	},
	
	searchCommand: function(state){
		var go = document.getElementById("search-go-button");
		var stop = document.getElementById("search-stop-button");
		var searchterm = document.getElementById("searchbar-textbox").value;
		if(searchterm != ""){
			if(state=="go"){
				go.setAttribute("hidden",true);
				stop.setAttribute("hidden",false);
				mdc.globalSearch(searchterm);
			}
			if(state=="stop"){
				go.setAttribute("hidden",false);
				stop.setAttribute("hidden",true);
				for(var i=0; i<search.worker.length; i++)
				{
					search.worker[i].terminate();
				}
				mdc.showThrobber('search', false);
				mdc.globalClear();
			}
		}
	},
	
	searchComplete: function(){
		mdc.showThrobber("search", false);
		var element = document.getElementById("search-go-button");
		element.setAttribute("mode","go");
	},
};

var tooltip={
	show: function(message, progress){
		
	}
};