/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Creative Commons
 * Attribution-NonCommercial-ShareAlike Version 3.0, as published
 * by Creative Commons. See 
 * http://creativecommons.org/licenses/by-nc-sa/3.0/legalcode for
 * more details. */ 
 
 /**
  * The main object of the radio window. Used for all radio-only actions
  */
 var radio={
	
	/**
	 * Gets the genre list from the shoutcast server. Used for displaying the genre picker
	 * @return a string array of the genres
	 */
	getGenreList: function(){
		var genrelist = mdc.getFile('http://yp.shoutcast.com/sbin/newxml.phtml');
		var xml = (new DOMParser()).
				parseFromString(genrelist, "text/xml");
		var genreList = xml.getElementsByTagName("genrelist")[0].
				getElementsByTagName("genre");
		return (genreList);
	},

	/**
	 * Fetches the station list for a specific URL (mostly the 
	 * ShoutCast Server + Genre or the ShoutCast Server + searchterm)
	 * @return array an array of the available stations
	 * @param url the url of the station list to get
	 */
	getStationList : function( url ) {
		var responseText = mdc.getFile(url);
		var xml = (new DOMParser()).
				parseFromString(responseText, "text/xml");
		var stationList = xml.getElementsByTagName("stationlist")[0].
				getElementsByTagName("station");
		return (stationList);
	},
	
	/**
	 * Creates the list box (Genre picker) in the Radio tab.
	 * @returns nothing
	 */
	createListBox: function(){
		var listbox = document.getElementById("radio_genremenupopup");
		if(!listbox.hasChildNodes()){
			mdc.showThrobber("radio", true);
			mdc.clearTree('radio_genremenupopup');
			var genreList = this.getGenreList();
			for (var i=0; i<genreList.length; i++) {
				var genre = genreList[i];
				var name = genre.getAttribute("name");
				var Item = document.createElement("menuitem");
					Item.setAttribute("value",name);
					Item.setAttribute("label",name);
				listbox.appendChild(Item);
			}
			mdc.showThrobber("radio", false);
		}
	},
	
	/** plays a ShoutCast stream. Notice the  url+="/;". It's used
	 * because the shoutCast servers send a HTTP response if the user agent
	 * is not WinAmp (?). the URL suffix fixes this :-)
	 * @param tree the ID of the tree to play the selected Element from
	 */
	playShoutCast: function( tree ){
		var tree = document.getElementById( tree );
		var cellIndex = 2;
		var id = tree.view.getCellText(tree.currentIndex, tree.columns.getColumnAt(cellIndex));
		var cellIndex = 1;
		var title = tree.view.getCellText(tree.currentIndex, tree.columns.getColumnAt(cellIndex));
		var url = this.readPlsFile("http://yp.shoutcast.com/sbin/tunein-station.pls?id="+id);
		url+="/;";
		//ShoutCast hack
		mdc.play( url, title, "shoutcast", id );
	},
	
	/**Lists all stations of the genre given
	 * @param genre the genre to list
	 */
	listStationOfGenre: function( genre ){
		mdc.showThrobber("radio",true);
		mdc.clearTree("radio_treechildren");
		var stationList = this.getStationList("http://yp.shoutcast.com/sbin/newxml.phtml?genre="+genre);
		this.writeTree(stationList);
		mdc.showThrobber("radio",false);	
	},
	
	getIdFromSelection: function( tree ){
		var tree = document.getElementById( tree );
		var start = {};
		var end = {};
		tree.view.selection.getRangeAt(0, start, end);//download all selected items
		var c = start.value;
		var id = tree.view.getItemAtIndex(c).getAttribute("id");
		return id;
	},
	
	/**
	 * Reads a PLS file (displays only the first entry!!!)
	 * @param url an URL where the pls file is located
	 * @return blergh the url of the music file or stream at position 1
	 */
	readPlsFile: function( url ){
		var pls = mdc.getFile( url );
		var pattern = /File1=(.*)/g;
		while (Ergebnis = pattern.exec(pls)){
			var blergh = Ergebnis[1];
		}
		return blergh;
	},
	
	fullSearch: function(searchterm){
		/*Makes a full Search through all genres on the ShoutCast server*/
		mdc.showThrobber( "search", true);
		//document.getElementById('search-stop').disabled = false;
		mdc.clearTree("radio_treechildren");
		searchterm = encodeURI(searchterm);
		var stationList = this.getStationList("http://yp.shoutcast.com/sbin/newxml.phtml?search="+searchterm);
		this.writeTree(stationList);
		mdc.searchComplete();
	},
	
	writeTree: function(stationList){
		for (var i=0; i<stationList.length; i++) {
			var station = stationList[i];
			var name = station.getAttribute("name");
			var mimeType = station.getAttribute("mt");
			var id = station.getAttribute("id");
			var bitrate = parseInt(station.getAttribute("br"));
			var currentTrack = station.getAttribute("ct");
			var numListeners = station.getAttribute("lc");
			var genres = station.getAttribute("genre").split(" ");
			var thisgenre = genres.shift();
			// Special case for 'top 40'
			if (thisgenre == "Top" && genres.length > 0 && genres[0] == "40")
				thisgenre += " 40";
			
			var tree = document.getElementById('radio_treechildren');
			var oItem = document.createElement("treeitem");

			var oRow = document.createElement("treerow");
			oItem.setAttribute("id", id+"%S"+currentTrack);
			
			var oCell = document.createElement("treecell");
			var starred = mdc.queryDatabase("SELECT COUNT(*) FROM 'bookmarks' WHERE id='"+id+"'",0);
			
			if(starred == "0"){
				oCell.setAttribute('value', 'false');
			}
			if(starred>0){
				oCell.setAttribute('value', 'true');
			}
			oRow.appendChild(oCell);
			
			var oCell = document.createElement("treecell");
			oCell.setAttribute('label', name);
			oCell.setAttribute('editable', 'false');
			oRow.appendChild(oCell);
					
			var oCell = document.createElement("treecell");		//hidden
			oCell.setAttribute('label', id);
			oCell.setAttribute('editable', 'false');
			oRow.appendChild(oCell);
		
			var oCell = document.createElement("treecell");
			oCell.setAttribute('label', numListeners);
			oCell.setAttribute('editable', 'false');
			oRow.appendChild(oCell);
				
			oItem.appendChild(oRow);
			tree.appendChild(oItem);
		}
	},
	
	getHeader: function( url, name ){
		dump('\nGetting '+name+' from '+url);
		var req = new XMLHttpRequest(); 
 	    req.open('GET', url, false); 
 	    req.send(null); 
 	    if(req.status == 200){ 
 	        var mimetype = req.getResponseHeader( name ); 
 	    }
		return mimetype;
	},
	
	updateBookmarks: function(event){
		/*
		############################
		#####   DANGER: BUGGY  #####
		############################
		Fix this as soon as possible
		*/
		var row = {}, column = {}, part = {};
		var tree = document.getElementById("radio_maintree");

		var boxobject = tree.boxObject;
		boxobject.QueryInterface(Components.interfaces.nsITreeBoxObject);
		boxobject.getCellAt(event.clientX, event.clientY, row, column, part);
		boxobject.getCellAt(event.clientX, event.clientY, row, column, part);
		
		if (column.value && typeof column.value != "string" && column.value.id=="stars"){
			var name = tree.view.getCellText(row.value, tree.columns.getColumnAt(1));
			var listeners = tree.view.getCellText(row.value, tree.columns.getColumnAt(3));
			var id = tree.view.getCellText(row.value, tree.columns.getColumnAt(2));			
			var url = this.readPlsFile("http://yp.shoutcast.com/sbin/tunein-station.pls?id="+id);
			url+="/;";	//ShoutCast hack
			var alreadyStarred = mdc.queryDatabase("SELECT COUNT(*) FROM 'bookmarks' WHERE id='"+id+"'", 0);
			if(alreadyStarred > 0){
				mdc.queryDatabase("DELETE FROM 'bookmarks' WHERE id='"+id+"'",0);
			}
			else{
				mdc.queryDatabase("INSERT INTO 'bookmarks' ('id', 'name', 'url', 'played') VALUES ('"+id+"','"+name.replace(/\'/gi,"''")+"','"+url+"','1')",0);
			}
		}
		document.getElementById("bookmarks_children").builder.rebuild();
	},
	
	getCurrentTrack: function(element){
		switch(mdc.currentType){
			case  "shoutcast":
				var stationName = element.value;
				var stationList = this.getStationList("http://yp.shoutcast.com/sbin/newxml.phtml?search="+stationName);
				for (var i=0; i<stationList.length;++i){
					var station = stationList[0];
					var currentTrack = station.getAttribute("ct");
					var id = station.getAttribute("id");
					if(id == mdc.currentId){
						mdc.selectSearchEngine('search');
						mdc.globalSearch(currentTrack);
						break;
					}
				}
				break;
			/*case "search":
				document.getElementById('searchbar-textbox').value = element.value;
				document.getElementById('tabsid').selectedIndex = '0';
				search.searchCommand()
				break;*/
		}
	},
};